// Configuration
const width = 1000;
const height = 600;
const radius = 60;

const parties = [
  {name: "Bloc Québécois", color: "#6ba7d9", abbreviation: "BQ"},
  {name: "Conservateur", color: "#194f99", abbreviation: "PCC"},
  {name: "Libéral", color: "#e9332f", abbreviation: "PLC"},
  {name: "Indépendant", color: "grey", abbreviation: "Ind."},
  {name: "Parti Vert", color: "#7bbd51", abbreviation: "PV"},
  {name: "NPD-Nouveau Parti Démocratique", color: "#f28135", abbreviation: "NPD"}
];

// Définition de l'échelle de couleurs à utiliser
const colors = d3.scaleOrdinal()
  .domain(parties.map(p => p.name))
  .range(parties.map(p => p.color));

// Chargement des données
Promise.all([
  d3.json('./data/canada.json'),
  d3.csv('./data/data.csv')
])
.then(initialize);

/**
 * Initialise la visualisation
 *
 * @param map       Les données au format TopoJSON à utiliser pour créer la carte
 * @param data      Les données associés aux résultats électoraux
 */
function initialize([map, data]) {
  // ***** Traitement des données *****

  // On transforme les données provenant du fichier CSV en une liste d'objets, où chaque objet représente
  // une circonscription quelconque.
  data = d3.nest()
    .key(d => d.id)
    .entries(data)
    .map(d => {
      const candidates = d.values.map(c => {
        return {
          name: c.candidate,
          votes: parseInt(c.votes),
          party: c.party
        }
      }).sort((a, b) => a.votes <= b.votes);
      return {
        id: parseInt(d.key),
        name: d.values[0].name,
        winnerParty: candidates[0].party,
        candidates: candidates
      }
    });

  // On transforme les données TopoJSON en données GeoJSON
  map = topojson.feature(map, map.objects.Canada);

  // ***** Initialisation du SVG et du zoom *****
  const zoom = d3.zoom()
    .scaleExtent([1, 20])
    .on("zoom", zoomed);

  const svg = d3.select('body')
    .append('svg')
    .attr('width', width)
    .attr('height', height)
    .call(zoom);

  // ***** Création de la carte *****
  const mapGroup = svg.append('g');

  // TODO
  // Initialiser une projection de type "d3.geoMercator" pour permettre l'affichage de la carte.
  // Vous pouvez consulter l'exemple vu un peu plus tôt pour vous aider à définir les différents paramètres.

  // TODO
  // Initialiser l'objet "d3.geoPath" à l'aide de la projection que vous avez définie un peu plus tôt.

  // TODO
  // Effectuer le rendu des polygones associés aux différentes circonscriptions. Assurez-vous d'effectuer ce
  // rendu à l'intérieur du groupe "mapGroup" afin que le zoom puisse fonctionner correctement.
  //
  // Voici les éléments à respecter:
  //
  // 1) Les polygones doivent être de la couleur du parti ayant remporté la circonscription.
  // 2) Lorsqu'un polygone est cliqué, celui-ci doit devenir sélectionné, c'est-à-dire changer de couleur (p. ex. #888).
  //    De plus, vous devez faire appel à la fonction "showResults" au même moment.
  // 3) Ajouter un élément de type "title" dans chacun des polygones qui permettra d'afficher le nom de la
  //    circonscription.

  // ***** Création du groupe affichant les résultats *****
  const resultsGroup = svg.append('g')
    .attr('transform', 'translate(25, 25)');

  resultsGroup.append('circle')
    .attr('cx', radius)
    .attr('cy', radius)
    .attr('r', radius * 1.2)
    .style('fill', '#fff')
    .style('fill-opacity', 0.95)
    .style('display', 'none');

  resultsGroup.append('g')
    .attr('transform', `translate(${radius}, ${radius})`);

  /**
   * Appelée à chaque fois qu'une circonscription est sélectionnée afin d'afficher les résultats associés à
   * cette même circonscription.
   *
   * @param divisionId    L'identifiant de la circonscription à utiliser.
   */
  function showResults(divisionId) {
    const currentData = data.find(d => d.id === divisionId);
    const pieChartGroup = resultsGroup.select('g');
    resultsGroup.select('circle').style('display', 'block');

    const arc = d3.arc()
      .innerRadius(0)
      .outerRadius(radius);

    const pies = (d3.pie()
      .value(d => d.votes))(currentData.candidates);

    // TODO
    // Tracer les pointes du pie chart à partir des données "pies". Assurez-vous de faire le
    // rendu du pie chart dans le groupe "pieChartGroup" et d'utiliser les fonctions "enter", "merge" et "update".
  }

  /**
   * Permet de mettre à jour les éléments de la carte lorsqu'un zoom est détecté.
   */
  function zoomed() {
    mapGroup.attr("transform", d3.event.transform);
    mapGroup.selectAll("path").style("stroke-width", 1 / d3.event.transform.k + "px");
  }
}

